package com.willalp.web.controller.app;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.willalp.clockingin.domain.BizLeave;
import com.willalp.clockingin.service.IBizLeaveService;
import com.willalp.common.core.controller.BaseController;
import com.willalp.common.core.domain.AjaxResult;
import com.willalp.common.core.page.TableDataInfo;
import com.willalp.common.utils.SecurityUtils;
import com.willalp.common.utils.StringUtils;
import com.willalp.files.domain.HsOrgPersonnelFiles;
import com.willalp.files.service.IHsOrgPersonnelFilesService;
import com.willalp.flow.domain.HsFlowTask;
import com.willalp.flow.domain.ReserveConsumingRecord;
import com.willalp.flow.domain.req.FlowReq;
import com.willalp.flow.service.HsFlowEngineService;
import com.willalp.flow.service.IBizLeaveImgService;
import com.willalp.flow.service.IReserveConsumingRecordService;
import com.willalp.flow.util.BeanMapUtilByReflect;
import com.willalp.msg.service.IHsAppUserMsgService;
import com.willalp.system.service.ISysConfigService;
import com.willalp.wx.service.WxMsgService;
import com.willalp.wx.utils.WxUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.Date;
import java.util.Map;

@Api("APP流控制器")
@RestController
@RequestMapping("/app")
public class AppFLowController extends BaseController {
    @Resource
    private IBizLeaveService bizLeaveService;

    @Resource
    private IBizLeaveImgService leaveImgService;

    @Resource
    private HsFlowEngineService flowEngineService;

    @Resource
    private ISysConfigService sysConfigService;

    @Resource
    private WxMsgService wxMsgService;

    @Resource
    private WxUtils wxUtils;

    @Resource
    private IHsAppUserMsgService userMsgService;

    /**
     * 请假提交
     *
     * @param leave
     * @return
     */
    @ApiOperation("提交请假")
    @PostMapping("/submitApply/leave")
    public AjaxResult submitLeave(@RequestBody BizLeave leave) {
        String idStr = IdWorker.getIdStr();
        String taskId = IdWorker.getIdStr();
        leave.setId(Long.parseLong(idStr));
        leave.setTaskId(taskId);
        leave.setApplyTime(new Date());
        if (leave.getImgUrls().size() != 0) {
            leave.getImgUrls().forEach(i -> i.setBizLeaveId(idStr));
            leaveImgService.saveBatch(leave.getImgUrls());
        }
        bizLeaveService.insertBizLeave(leave);
        Map<String, Object> map = null;
        try {
            //对象转换为Map集合
            map = BeanMapUtilByReflect.beanToMap(leave);
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        }
        String bizLeaveFlowRunId = sysConfigService.selectConfigByKey(HsFlowEngineService.BIZ_LEAVE_FLOW_KEY);
        return AjaxResult.success(
                flowEngineService.submitATask(new HsFlowTask(taskId, idStr, bizLeaveFlowRunId), map));
    }

    @Resource
    private IReserveConsumingRecordService reserveConsumingRecordService;

    /**
     * 物资领用提交
     *
     * @param reserveConsumingRecord
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @ApiOperation("物资领用提交")
    @PostMapping("/submitApply/rcr")
    public AjaxResult submitLeave(@RequestBody ReserveConsumingRecord reserveConsumingRecord) {
        String idStr = IdWorker.getIdStr();
        String taskId = IdWorker.getIdStr();
        reserveConsumingRecord.setId(idStr);
        reserveConsumingRecord.setApplyUserId(SecurityUtils.getUsername());
        reserveConsumingRecord.setTitle("物资领用审批");
        reserveConsumingRecordService.save(reserveConsumingRecord);
        Map<String, Object> map = null;
        try {
            //对象转换为Map集合
            map = BeanMapUtilByReflect.beanToMap(reserveConsumingRecord);
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        }
        String reserveConsumingRunId = sysConfigService.selectConfigByKey(HsFlowEngineService.RESERVE_FLOW_KEY);
        return AjaxResult.success(
                flowEngineService.submitATask(new HsFlowTask(taskId, idStr, reserveConsumingRunId), map));
    }

    /**
     * 下一步
     *
     * @param fr
     * @return
     */
    @ApiOperation("下一步")
    @PostMapping("/nextStep")
    public AjaxResult nextStep(@RequestBody FlowReq fr) {
        return AjaxResult.success(
                flowEngineService.nextStep(fr));
    }

    /**
     * 获取待办
     *
     * @param taskId
     * @param request
     * @return
     */
    @ApiOperation("获取指定待办任务")
    @RequestMapping("/gtdToId")
    public AjaxResult getToDoByTaskId(String taskId, HttpServletRequest request) {
        return AjaxResult.success(
                flowEngineService.getToDoByTaskId(taskId));
    }

    /**
     * 获取
     *
     * @param taskId
     * @param request
     * @return
     */
    @ApiOperation("获取指定待办任务")
    @RequestMapping("/gBlToId")
    public AjaxResult getLeaveByTaskId(String taskId, HttpServletRequest request) {
        return AjaxResult.success(
                bizLeaveService.getBaseMapper().selectOne(new QueryWrapper<BizLeave>()
                        .eq("task_id", taskId)));
    }

    /**
     * 获取待办
     *
     * @param request
     * @return
     */
    @ApiOperation("获取待办")
    @RequestMapping("/gtd")
    public TableDataInfo getToDo(HttpServletRequest request) {
        return getDataTable(
                flowEngineService.getToDo(request.getHeader("UserName")));
    }

    /**
     * 获取已办
     *
     * @param request
     * @return
     */
    @ApiOperation("获取已办")
    @RequestMapping("/gd")
    public TableDataInfo getDone(HttpServletRequest request) {
        return getDataTable(
                flowEngineService.getDone(request.getHeader("UserName")));
    }

    /**
     * 获取所有发起审批
     */
    @GetMapping("/initiator")
    public TableDataInfo initiatorList(HttpServletRequest request) {
        startPage();
        return getDataTable(
                flowEngineService.getInitiator(request.getHeader("UserName")));
    }

    /**
     * 获取当前任务记录
     *
     * @param taskId
     * @param request
     * @return
     */
    @ApiOperation("获取当前任务进度")
    @RequestMapping("/ctr")
    public AjaxResult currentTaskRecords(String taskId, HttpServletRequest request) {
        return AjaxResult.success(
                flowEngineService.currentTaskRecords(taskId));
    }

    /**
     * 驳回时选择进度
     *
     * @param taskId
     * @param request
     * @return
     */
    @ApiOperation("驳回时选择进度")
    @RequestMapping("/reject/ctr")
    public AjaxResult rejectTaskRecords(String taskId, HttpServletRequest request) {
        return AjaxResult.success(
                flowEngineService.rejectTaskRecords(taskId));
    }


    /**
     * 任务撤销
     *
     * @param taskId
     * @return
     */
    @ApiOperation("任务撤销")
    @RequestMapping("/revoke")
    public AjaxResult revoke(String taskId) {
        return toAjax(
                flowEngineService.revoke(taskId));
    }

    @Resource
    private IHsOrgPersonnelFilesService orgPersonnelFilesService;

    /**
     * 请假记录
     *
     * @return
     */
    @ApiOperation("请假记录")
    @RequestMapping("/leaveList")
    public TableDataInfo leaveList() {
        String username = SecurityUtils.getUsername();
        if (StringUtils.isNotBlank(username)) {
            long department = orgPersonnelFilesService.count(new QueryWrapper<HsOrgPersonnelFiles>()
                    .eq("department", "319")
                    .eq("user_code", username));
            if (department > 0) {
                startPage();
                return getDataTable(
                        bizLeaveService.selectPersonnelList(new BizLeave()));
            }
        }
        return getDataTable(new ArrayList<>());
    }
}
