package com.willalp.flow.service.impl;

import cn.hutool.json.JSONObject;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.willalp.common.exception.base.BaseException;
import com.willalp.common.utils.DateUtils;
import com.willalp.common.utils.SecurityUtils;
import com.willalp.common.utils.StringUtils;
import com.willalp.flow.domain.*;
import com.willalp.flow.domain.enums.FlowEnum;
import com.willalp.flow.domain.json.Operators;
import com.willalp.flow.domain.req.FlowReq;
import com.willalp.flow.mapper.*;
import com.willalp.flow.service.*;
import com.willalp.msg.domain.HsAppUserMsg;
import com.willalp.msg.service.IHsAppUserMsgService;
import com.willalp.system.domain.vo.PostTreeVo;
import com.willalp.system.service.ISysConfigService;
import com.willalp.wx.domain.WxMsgTemplateApproval;
import com.willalp.wx.domain.WxMsgTemplateApprovalResult;
import com.willalp.wx.enums.WxTemplateKeyEnum;
import com.willalp.wx.service.WxMsgService;
import com.willalp.wx.utils.WxUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

@Service
public class HsFlowEngineServiceImpl implements HsFlowEngineService {

    @Resource
    private HsFlowBaseMapper hsFlowBaseMapper;

    @Resource
    private IHsFlowBaseService flowBaseService;

    @Resource
    private HsFlowExamplesMapper hsFlowExamplesMapper;

    @Resource
    private HsFlowModelMapper hsFlowModelMapper;

    @Resource
    private IHsFlowRunningRecordService flowRunningRecordService;

    @Resource
    private HsFlowTaskMapper hsFlowTaskMapper;

    @Resource
    private HsFlowConditionModelMapper flowConditionModelMapper;

    @Resource
    private IHsFlowConditionModelService conditionModelService;

    @Resource
    private IBizLeaveImgService leaveImgService;

    @Resource
    private WxMsgService wxMsgService;

    @Resource
    private WxUtils wxUtils;

    @Resource
    private IHsAppUserMsgService userMsgService;

    @Resource
    private IHsFlowTaskService taskService;

    @Resource
    private ISysConfigService sysConfigService;


    @Override
    public Map<String, Object> submitATask(HsFlowTask task, Map<String, Object> params) {
        String examplesId = task.getExamplesId();
        if (StringUtils.isBlank(examplesId)) {
            throw new BaseException("EXAMPLES_ID - IS_NULL");
        }
        HsFlowExamples hsFlowExamples = hsFlowExamplesMapper.selectById(examplesId);
        if (null == hsFlowExamples) {
            throw new BaseException("流程实例为定义或找不到");
        }
        HsFlowBase hsFlowBase = hsFlowBaseMapper.firstFlowBase(examplesId);
        if (null == hsFlowBase) {
            throw new BaseException("请创建实例");
        }
        //判断是否存在上级节点 不存在则为父级组件 然后跳过
        if (StringUtils.isBlank(hsFlowBase.getUpStepId())) {
            if (StringUtils.isNotBlank(hsFlowBase.getDownStepId())) {
                hsFlowBase = hsFlowBaseMapper.selectById(hsFlowBase.getDownStepId());
            } else {
                throw new BaseException("流程配置错误");
            }
        }
        //判断提交前置条件，如果有更新至对应组件
        hsFlowBase = getFlowBaseByTermResult(task.getId(), hsFlowBase, params);
        if (StringUtils.isBlank(hsFlowBase.getDownStepId())) {
            task.setIsCandidate("1");
        }
        task.setInitiator(SecurityUtils.getUsername());
        task.setBaseGroupId(hsFlowBase.getGroupId());
        task.setBaseId(hsFlowBase.getId());
        task.setBaseName(hsFlowBase.getFlowModuleName());
        task.setTaskName(hsFlowExamples.getExamplesName());
        task.setCreateTime(new Date());
        //获取候选人
        if (ObjectUtils.isNotNull(params)) {
            String userName = (String) params.get("candidate");
            if (StringUtils.isNotBlank(userName)) {
                task.setOperator(userName);
            }
        }
        try {
            hsFlowTaskMapper.insert(task);
        } catch (Exception e) {
            e.printStackTrace();
            throw new BaseException("提交失败");
        }
        //启动超时删除线程
        taskService.removeWaitTimeOutTask(task);
        Map<String, Object> map = new HashMap<>(4);
        map.put("taskId", task.getId());
        map.put("tree", getApprovalTreePost(task.getBaseGroupId(), task.getBaseId()));
        return map;
    }

    @Override
    public int submitCandidate(String taskId, String candidate, String applyUserName) {
        HsFlowTask hsFlowTask = hsFlowTaskMapper.selectById(taskId);
        if (hsFlowTask == null) {
            throw new BaseException("超时未提交任务已过期");
        }
        String content = null;
        try {
            //生成审批结果消息列表
            String examplesId = hsFlowTask.getExamplesId();
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            HsFlowExamples example = hsFlowExamplesMapper.selectById(examplesId);
            Map<String, Object> formData = hsFlowTaskMapper
                    .selectBusinessCase(example.getBusinessTableName(), hsFlowTask.getBusinessId());
            if (examplesId.equals(sysConfigService.selectConfigByKey(HsFlowEngineService.BIZ_LEAVE_FLOW_KEY))) {
                String leaveStartTime = DateUtils.dateTime(sdf.parse(formData.get("leave_start_time").toString()));
                String leaveEndTime = DateUtils.dateTime(sdf.parse(formData.get("leave_end_time").toString()));
                content = "<p>标题：" + formData.get("title") + "</p>" +
                        "<p>申请人：" + formData.get("apply_user_name") + "</p>" +
                        "<p>开始时间：" + leaveStartTime + "</p>" +
                        "<p>结束时间：" + leaveEndTime + "</p>";
            } else if (examplesId.equals(sysConfigService.selectConfigByKey(HsFlowEngineService.FILE_APPROVAL_FLOW_KEY))) {
                //异步生成审批消息列表
                content = "<p>标题：" + formData.get("title") + "</p>" +
                        "<p>申请人：" + formData.get("apply_user_name") + "</p>";
            } else if (examplesId.equals(sysConfigService.selectConfigByKey(HsFlowEngineService.RESERVE_FLOW_KEY))) {
                content = "<p>标题：" + formData.get("title") + "</p>" +
                        "<p>申请人：" + formData.get("apply_user_name") + "</p>" +
                        "<p>物资名称：" + formData.get("reserve_name") + "</p>" +
                        "<p>申请数量：" + formData.get("apply_amount") + "</p>";
            }
            //异步生成审批消息列表
            userMsgService.saveApprovalMsgAndPush("1",
                    String.valueOf(formData.get("title")), content, taskId,
                    candidate, applyUserName, new Date());
        } catch (ParseException e) {
            e.printStackTrace();
        }
        if (StringUtils.isNotBlank(candidate)) {
            hsFlowTask.setOperator(candidate);
            if (0 < hsFlowTaskMapper.updateById(hsFlowTask)) {
                return 1;
            }
        }
        return 0;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public int nextStep(FlowReq flow) {
        if (StringUtils.isBlank(flow.getTaskId())) {
            throw new BaseException("FLOW_TASK_ID - IS_NULL");
        }
        if (StringUtils.isBlank(flow.getResult())) {
            throw new BaseException("FLOW_RESULT - IS_NULL");
        }
        //获取任务实例
        HsFlowTask hsFlowTask = hsFlowTaskMapper.selectById(flow.getTaskId());
        if (null == hsFlowTask) {
            throw new BaseException("没有找到任务实例");
        }
        if ("1".equals(hsFlowTask.getIsEnd())) {
            throw new BaseException("任务已结束");
        }
        if (StringUtils.isBlank(flow.getBaseId())) {
            throw new BaseException("FLOW_BASE_ID - IS_NULL");
        }
        if (StringUtils.isBlank(hsFlowTask.getBaseGroupId())) {
            throw new BaseException("FLOW_BASE_GROUP_ID - IS_NULL");
        }
        //获取当前基础组件
        HsFlowBase hsFlowBase = flowBaseService.getCurrentFlowTaskBase(hsFlowTask.getBaseGroupId(), flow.getBaseId());
        if (null == hsFlowBase) {
            throw new BaseException("没有找到组件");
        }
        //判断是否存在上级节点 不存在则为父级组件 然后跳过
        if (StringUtils.isBlank(hsFlowBase.getUpStepId())) {
            if (StringUtils.isNotBlank(hsFlowBase.getDownStepId())) {
                hsFlowBase = hsFlowBaseMapper.selectById(hsFlowBase.getDownStepId());
            } else {
                throw new BaseException("流程配置错误");
            }
        }
        if (!SecurityUtils.getUsername().equals(hsFlowTask.getOperator())) {
            throw new BaseException("无操作权限");
        }
        if (FlowEnum.IS_ONE.getValue().equals(hsFlowTask.getIsEnd())) {
            throw new BaseException("任务已经结束");
        }
        String stepId = null;
        //获取是否 是同意结果
        boolean resultTrue = FlowEnum.RESULT_TRUE.getValue().equals(flow.getResult());
        boolean resultCaseOf = FlowEnum.RESULT_CASE_OF.getValue().equals(flow.getResult());
        Map<String, Object> formData = flow.getFormData();
        List<HsFlowRunningRecord> list = new ArrayList<>();
        //生成记录
        HsFlowRunningRecord frr = new HsFlowRunningRecord();
        frr.setBaseId(flow.getBaseId());
        frr.setTaskId(flow.getTaskId());
        frr.setCreateTime(new Date());
        if (resultCaseOf) {
            //转交
            hsFlowTask.setOperator(flow.getNextApproval());
            frr.setComment("转交至 " + flow.getNextUserName());
            frr.setOperator(SecurityUtils.getUsername());
            frr.setYesOrNo(flow.getResult());
        } else {
            //查看有无条件组件
            if (resultTrue) {
                hsFlowBase = getFlowBaseByTermResult(flow.getTaskId(), hsFlowBase, formData);
            }
            if (resultTrue) {
                stepId = hsFlowBase.getDownStepId();
            } else {
                // stepId = hsFlowBase.getUpStepId();
                stepId = flow.getRejectBaseId();
            }
            String username = SecurityUtils.getUsername();
            frr.setComment(hsFlowTask.getBaseName() + "：" + SecurityUtils.getLoginUser().getUser().getNickName() + " - " + flow.getComment());
            frr.setOperator(username);
            frr.setYesOrNo(flow.getResult());
        }
        list.add(frr);
        if (StringUtils.isNotBlank(stepId)) {
            hsFlowTask.setBaseId(stepId);
            //判断新组件下一步是否需要选择人员
            HsFlowBase hsFlowBase1 = flowBaseService.getCurrentFlowTaskBase(hsFlowTask.getBaseGroupId(), stepId);
            if (StringUtils.isNotBlank(flow.getNextApproval())) {
                frr.setOperator(SecurityUtils.getUsername());
                hsFlowTask.setOperator(flow.getNextApproval());
                hsFlowTask.setBaseName(hsFlowBase1.getFlowModuleName());
            } else if (!resultTrue) {
                hsFlowTask.setBaseId(stepId);
                hsFlowTask.setOperator(flow.getNextApproval());
                HsFlowBase currentFlowTaskBase = flowBaseService.getCurrentFlowTaskBase(hsFlowTask.getBaseGroupId(), stepId);
                hsFlowTask.setBaseName(currentFlowTaskBase.getFlowModuleName());
                //拒绝时 通过记录 反向比对 查找上一步 同意的数据并 更新任务
//                List<HsFlowRunningRecord> flowRunningRecords = flowRunningRecordService.selectHsFlowRunningRecordTrue(flow.getTaskId());
//                int count = 0;
//                for (int i = 0; i < flowRunningRecords.size(); i++) {
//                    String baseId = flowRunningRecords.get(i).getBaseId();
//                    if (StringUtils.isNotBlank(baseId) && StringUtils.equals(baseId, stepId)) {
//                        hsFlowTask.setBaseId(flowRunningRecords.get(i).getBaseId());
//                        hsFlowTask.setOperator(flowRunningRecords.get(i).getOperator());
//                        HsFlowBase currentFlowTaskBase = flowBaseService.getCurrentFlowTaskBase(hsFlowTask.getBaseGroupId(), baseId);
//                        hsFlowTask.setBaseName(currentFlowTaskBase.getFlowModuleName());
//                        count++;
//                        break;
//                    }
//                }
                //没有找到 直接结束
//                if (count == 0) {
//                    stepId = null;
//                }
            } else {
                //避免异常找不到任务默认admin
                hsFlowTask.setOperator("admin");
                throw new BaseException("未选择下一步审批人员");
            }
            if (null != hsFlowBase1 && StringUtils.isBlank(hsFlowBase1.getDownStepId()) &&
                    StringUtils.isBlank(hsFlowBase1.getTermModuleId())) {
                hsFlowTask.setIsCandidate("1");
            } else {
                hsFlowTask.setIsCandidate("0");
            }
        }
        //任务结束
        if (StringUtils.isBlank(stepId) && !resultCaseOf) {
            hsFlowTask.setBaseId(null);
            hsFlowTask.setOperator(hsFlowTask.getOperator());
            hsFlowTask.setIsEnd("1");
            hsFlowTask.setRemark("任务已结束");
            HsFlowRunningRecord frr1 = new HsFlowRunningRecord();
            frr1.setComment("自动结束");
            frr1.setOperator(SecurityUtils.getUsername());
            frr1.setTaskId(flow.getTaskId());
            frr1.setCreateTime(new Date(System.currentTimeMillis() + 1000));
            list.add(frr1);
            //生成审批结果消息列表
            HsAppUserMsg appUserMsg = new HsAppUserMsg();
            appUserMsg.setMsgContent("您的 [" + formData.get("title") + "] 申请" + (resultTrue ? "已通过" : "已拒绝"));
            appUserMsg.setMsgTitle("审批结果");
            appUserMsg.setMsgIsRead("0");
            appUserMsg.setUserCode(String.valueOf(formData.get("apply_user_id")));
            appUserMsg.setMsgType("4");
            appUserMsg.setRemark(hsFlowTask.getId());
            appUserMsg.setCreateTime(new Date());
            userMsgService.save(appUserMsg);

            //推送微信审批结果通知
            WxMsgTemplateApprovalResult wxMsgTemplateApproval = new WxMsgTemplateApprovalResult();
            wxMsgTemplateApproval.setPhrase4(wxUtils.getFormat(resultTrue ? "已通过" : "已拒绝"));
            wxMsgTemplateApproval.setDate3(wxUtils.getFormat(DateUtils.parseDateToStr("yyyy-MM-dd HH:mm:ss", new Date())));
            wxMsgTemplateApproval.setThing6(wxUtils.getFormat(String.valueOf(formData.get("title"))));
            wxMsgService.sendQRCodeMsg(String.valueOf(formData.get("apply_user_id")), wxMsgTemplateApproval, WxTemplateKeyEnum.APPROVAL_RESULT, true);
        } else {
            //任务继续
            hsFlowTask.setMsgIsRead("0");
            //生成审批结果消息列表
            HsAppUserMsg appUserMsg = new HsAppUserMsg();
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            String content = null;
            try {
                String examplesId = hsFlowTask.getExamplesId();
                if (examplesId.equals(sysConfigService.selectConfigByKey(HsFlowEngineService.BIZ_LEAVE_FLOW_KEY))) {
                    content = "<p>标题：" + formData.get("title") + "</p>" +
                            "<p>申请人：" + formData.get("apply_user_name") + "</p>" +
                            "<p>开始时间：" + DateUtils.dateTime(sdf.parse(formData.get("leave_start_time").toString())) + "</p>" +
                            "<p>结束时间：" + DateUtils.dateTime(sdf.parse(formData.get("leave_end_time").toString())) + "</p>";
                } else if (examplesId.equals(sysConfigService.selectConfigByKey(HsFlowEngineService.FILE_APPROVAL_FLOW_KEY))) {
                    //异步生成审批消息列表
                    content = "<p>标题：" + formData.get("title") + "</p>" +
                            "<p>申请人：" + formData.get("apply_user_name") + "</p>";
                } else if (examplesId.equals(sysConfigService.selectConfigByKey(HsFlowEngineService.RESERVE_FLOW_KEY))) {
                    content = "<p>标题：" + formData.get("title") + "</p>" +
                            "<p>申请人：" + formData.get("apply_user_name") + "</p>" +
                            "<p>物资名称：" + formData.get("reserve_name") + "</p>" +
                            "<p>申请数量：" + formData.get("apply_amount") + "</p>";
                }
            } catch (ParseException e) {
                e.printStackTrace();
            }
            appUserMsg.setMsgContent(content);
            appUserMsg.setMsgTitle(String.valueOf(formData.get("title")));
            appUserMsg.setMsgIsRead("0");
            appUserMsg.setUserCode(hsFlowTask.getOperator());
            appUserMsg.setMsgType("1");
            appUserMsg.setRemark(hsFlowTask.getId());
            appUserMsg.setCreateTime(new Date());
            userMsgService.save(appUserMsg);
            //推送微信待审批通知
            WxMsgTemplateApproval wxMsgTemplateApproval = new WxMsgTemplateApproval();
            wxMsgTemplateApproval.setThing5(wxUtils.getFormat(String.valueOf(formData.get("title"))));
            wxMsgTemplateApproval.setThing11(wxUtils.getFormat(String.valueOf(formData.get("apply_user_name"))));
            wxMsgTemplateApproval.setTime12(wxUtils.getFormat(DateUtils.parseDateToStr("yyyy-MM-dd HH:mm:ss", new Date())));
            wxMsgService.sendQRCodeMsg(hsFlowTask.getOperator(), wxMsgTemplateApproval, WxTemplateKeyEnum.APPROVAL_WAIT, true);
        }
        hsFlowTaskMapper.updateById(hsFlowTask);
        flowRunningRecordService.saveBatch(list);
        return 1;
    }


    /**
     * 通过条件组件判断结果获取流程组件
     *
     * @param hsFlowBase
     * @param params
     * @return
     */
    private HsFlowBase getFlowBaseByTermResult(String taskId, HsFlowBase hsFlowBase, Map<String, Object> params) {
        //判断当前组件有无条件组件
        if (StringUtils.isNotBlank(hsFlowBase.getTermModuleId()) &&
                null != params) {
            //遍历绑定的条件组件 查找下一步 （待添加条件级别优化）
            List<HsFlowConditionModel> hsFlowConditionModels = flowConditionModelMapper.queryConditionGroup(
                    hsFlowBase.getUpStepId(), hsFlowBase.getTermModuleId());
            for (HsFlowConditionModel item : hsFlowConditionModels) {
                String paramName = item.getParamName();
                String[] s = paramName.split("_");
                StringBuilder appendStr = new StringBuilder();
                if (0 != s.length) {
                    appendStr.append(s[0]);
                    for (int i = 1; i < s.length; i++) {
                        appendStr.append(s[i].substring(0, 1).toUpperCase()).append(s[i].substring(1));
                    }
                    paramName = appendStr.toString();
                }
                if (null != params.get(paramName)) {
                    if (conditionModelService.executeConditionJudgment(item, params.get(paramName))) {
                        //对应条件组件
                        HsFlowBase termModule = hsFlowBaseMapper.selectOne(
                                new QueryWrapper<HsFlowBase>().eq("term_module_id", item.getId()));
                        if (null == termModule) {
                            throw new BaseException("流程条件配置出错");
                        }
                        //通过条件组件获取审批组件
                        if (StringUtils.isBlank(termModule.getDownStepId())) {
                            throw new BaseException("流程条件配置出错，没有后续审批人");
                        }
                        HsFlowBase taskCurrentBase = flowBaseService.getCurrentFlowTaskBase(termModule.getGroupId()
                                , termModule.getDownStepId());
                        if (null == taskCurrentBase) {
                            throw new BaseException("流程条件配置出错");
                        }
                        return taskCurrentBase;
                    }
                }
            }
            hsFlowTaskMapper.deleteById(taskId);
            flowRunningRecordService.remove(new QueryWrapper<HsFlowRunningRecord>()
                    .eq("task_id", taskId));
            throw new BaseException("流程条件未满足，流程关闭，请重新发起");
        } else {
            return hsFlowBase;
        }
    }

    @Override
    public List<HsFlowTask> getToDoByTaskId(String taskId) {
        if (StringUtils.isBlank(taskId)) {
            return null;
        }
        List<HsFlowTask> hsFlowTasks = hsFlowTaskMapper.selectList(new QueryWrapper<HsFlowTask>()
                .eq("id", taskId));
        return getHsFlowTasks(hsFlowTasks);
    }

    @Override
    public List<HsFlowTask> getToDo(String userCode) {
        if (StringUtils.isBlank(userCode)) {
            return null;
        }
        List<HsFlowTask> hsFlowTasks = hsFlowTaskMapper.selectAllByOperator(userCode);
        return getHsFlowTasks(hsFlowTasks);
    }

    @Override
    public List<HsFlowTask> getInitiator(String userCode) {
        if (StringUtils.isBlank(userCode)) {
            return null;
        }
        List<HsFlowTask> hsFlowTasks = hsFlowTaskMapper.selectAllByInitiator(userCode);
        return getHsFlowTasks(hsFlowTasks);
    }

    @Override
    public List<HsFlowTask> getDone(String userCode) {
        if (StringUtils.isBlank(userCode)) {
            return null;
        }
        List<HsFlowTask> hsFlowTasks = hsFlowTaskMapper.getDone(userCode);
        return getHsFlowTasks(hsFlowTasks);
    }

    private List<HsFlowTask> getHsFlowTasks(List<HsFlowTask> hsFlowTasks) {
        if (0 != hsFlowTasks.size()) {
            List<HsFlowExamples> hsFlowExamples = hsFlowExamplesMapper.selectHsFlowExamplesList(null);
            hsFlowTasks.forEach(item -> {
                String businessId = item.getBusinessId();
                if (0 != hsFlowExamples.size()) {
                    hsFlowExamples.forEach(example -> {
                        if (StringUtils.isNotBlank(businessId)) {
                            Map<String, Object> stringObjectMap = hsFlowTaskMapper
                                    .selectBusinessCase(example.getBusinessTableName(), businessId);
                            if (ObjectUtils.isNotNull(stringObjectMap)) {
                                if ("biz_leave".equals(example.getBusinessTableName())) {
                                    List<Map<String, Object>> leaveImgs = leaveImgService.getBaseMapper().selectMaps(new QueryWrapper<BizLeaveImg>().eq("biz_leave_id", businessId));
                                    stringObjectMap.put("imgUrls", leaveImgs);
                                }
                                //统一时间类型字段处理
                                Set<String> strings = stringObjectMap.keySet();
                                DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
                                strings.forEach(str -> {
                                    if (StringUtils.isNotBlank(str) && str.contains("time") || str.contains("date")) {
                                        if (stringObjectMap.get(str) instanceof LocalDateTime) {
                                            stringObjectMap.put(str, formatter.format((LocalDateTime) stringObjectMap.get(str)));
                                        }
                                    }
                                });
                                item.setFormData(new JSONObject(stringObjectMap));
                            }
                        }
                    });
                }
            });
        }
        return hsFlowTasks;
    }

    @Override
    public List<HsFlowRunningRecord> currentTaskRecords(String taskId) {
        if (StringUtils.isNotBlank(taskId)) {
            HsFlowRunningRecord hsFlowRunningRecord = new HsFlowRunningRecord();
            hsFlowRunningRecord.setTaskId(taskId);
            return flowRunningRecordService.selectHsFlowRunningRecordList(hsFlowRunningRecord);
        }
        return null;
    }

    @Override
    public List<HsFlowRunningRecord> rejectTaskRecords(String taskId) {
        if (StringUtils.isNotBlank(taskId)) {
            List<HsFlowRunningRecord> flowRunningRecords = flowRunningRecordService.getBaseMapper().selectList(new QueryWrapper<HsFlowRunningRecord>()
                    .eq("task_id", taskId).last("AND yes_or_no = 'RESULT_TRUE' GROUP BY base_id"));
            flowRunningRecords.forEach(flowRunningRecord -> {
                String comment = flowRunningRecord.getComment();
                flowRunningRecord.setComment(comment.substring(0, comment.indexOf("-")));
            });
            return flowRunningRecords;
        }
        return null;
    }

    @Override
    public int revoke(String taskId) {
        if (StringUtils.isNotBlank(taskId)) {
            long selectCount = flowRunningRecordService.getBaseMapper().selectCount(new QueryWrapper<HsFlowRunningRecord>()
                    .eq("task_id", taskId));
            if (selectCount > 0) {
                throw new BaseException("已有审批结果无法撤销");
            }
            return hsFlowTaskMapper.deleteById(taskId);
        }
        return 0;
    }

    @Override
    public int removeBusinessForm(String examplesId, String businessId) {
        if (!(StringUtils.isNotBlank(examplesId) && StringUtils.isNotBlank(businessId))) {
            throw new BaseException("关键参数为空无法删除业务表单");
        }
        HsFlowExamples hsFlowExamples = hsFlowExamplesMapper.selectById(examplesId);
        if (null == hsFlowExamples) {
            throw new BaseException("FLOW_EXAMPLES_NUll");
        }
        String businessTableName = hsFlowExamples.getBusinessTableName();
        if (StringUtils.isNotBlank(businessTableName)) {
            return hsFlowExamplesMapper.removeBusinessForm(businessTableName, businessId);
        }
        return 0;
    }

    @Override
    public List<PostTreeVo> getApprovalTreePost(String groupId, String baseId) {
        HsFlowBase flowTaskCurrentBase = flowBaseService.getCurrentFlowTaskBase(groupId, baseId);
        if (null == flowTaskCurrentBase) {
            return null;
        }
        PostTreeVo postTreeVo = new PostTreeVo();
        postTreeVo.setText(flowTaskCurrentBase.getFlowModuleName());
        postTreeVo.setValue(flowTaskCurrentBase.getId());
        Operators operators = JSON.parseObject(flowTaskCurrentBase.getOperator(), Operators.class);
        List<Operators.OperatorDto> assignedUser = operators.getAssignedUser();
        List<PostTreeVo> list = new ArrayList<>();
        if (0 != assignedUser.size()) {
            List<PostTreeVo> postTreeCcVos = new ArrayList<>();
            assignedUser.forEach(item -> {
                PostTreeVo postTreeCcVo = new PostTreeVo();
                postTreeCcVo.setText(item.getName());
                postTreeCcVo.setValue(item.getId());
                postTreeCcVos.add(postTreeCcVo);
            });
            if (postTreeCcVos.size() != 0) {
                postTreeVo.setChildren(postTreeCcVos);
            } else {
                postTreeVo.setChildren(null);
            }
            list.add(postTreeVo);
        }
        return list;
    }

    @Override
    public List<PostTreeVo> getNextApprovalTreePost(String groupId, String baseId) {
        HsFlowBase nextFlowTaskBase = flowBaseService.getNextFlowTaskBase(groupId, baseId);
        if (null == nextFlowTaskBase) {
            return null;
        }
        return getApprovalTreePost(groupId, nextFlowTaskBase.getId());
    }
}

